package baka.dataflow;

import baka.gui.Datovod;
import baka.gui.DatovodPoint;
import baka.gui.GraphicBlock;
import baka.gui.GraphicCanvas;
import baka.main.Config;
import java.io.File;
import java.util.LinkedList;
import java.util.Queue;
import java.util.Vector;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 *
 * @author madda
 */
public class Conversion {

    Vector<GraphicBlock> gbloks;
    Vector<DataBlock> dataBlocks;
    Vector<Datovod> datovods;
    public int conversionType = 0; // 0 - synchronna

    public Conversion(Vector<GraphicBlock> gbloks, Vector<Datovod> datovods, int conversionType) {
        this.datovods = new Vector<Datovod>();
        this.gbloks = new Vector<GraphicBlock>();
        this.gbloks = gbloks;
        this.datovods = datovods;
        this.conversionType = conversionType;
    }

    private void extractDataBlocks() {
        this.dataBlocks = new Vector<DataBlock>();
        for (int i = 0; i < gbloks.size(); i++) {
            //System.out.println("datablock : " + gbloks.elementAt(i).getDataBlock());
            DataBlock temp = gbloks.elementAt(i).getDataBlock();
            temp.resetBlockState();

            this.dataBlocks.add(temp);
        }
        for (int i = 0; i < datovods.size(); i++) {
            //System.out.println("datovods : " + datovods.elementAt(i));
            Datovod temp = datovods.elementAt(i);
            int startIndex = temp.getStartPoint().getBlockId();
            int startInputId = temp.getStartPoint().getOutputId();
            int endIndex = temp.getEndPoint().getBlockId();
            int endOutputId = temp.getEndPoint().getInputId();
            DataChannel current = new DataChannel(temp.getType());
            //System.out.println("from:" + startIndex + " to:" + endIndex + " fromId:" + startInputId + " toId:" + endOutputId);
            dataBlocks.elementAt(startIndex).outputBus.addChannel(startInputId, current);
            dataBlocks.elementAt(endIndex).inputBus.addChannel(endOutputId, current);
        }
    }

    public Conversion(File file, GraphicCanvas canvas) {
        this.dataBlocks = new Vector<DataBlock>();
        this.datovods = new Vector<Datovod>();
        this.gbloks = new Vector<GraphicBlock>();
        loadConversion(file, canvas);
    }

    public void executeConversion() {
        extractDataBlocks();
        if (this.conversionType == 0) {
            // synchronna konverzia
            //System.out.println("vykonavam synchronnu konverziu");
            int lastExecuted = 1;
            while (lastExecuted > 0) {
                lastExecuted = 0;
                for (int i = 0; i < dataBlocks.size(); i++) {
                    if (dataBlocks.elementAt(i).execute()) {
                        lastExecuted++;
                    }
                }
                for (int i = 0; i < dataBlocks.size(); i++) {
                    dataBlocks.elementAt(i).sendOutputs();
                }
                //System.out.println("In last round was executed : " + lastExecuted + " bloks.");
            }
            for (int i = 0; i < dataBlocks.size(); i++) {
                dataBlocks.elementAt(i).finalizeBlockWork();
            }

        } else {
            // nesynchronna
            //System.out.println("Execute asynchron conversion.");

            //zaraď všetky “source“ bloky na front spracovania
            Queue<DataBlock> front = new LinkedList<DataBlock>();
            for (int i = 0; i < dataBlocks.size(); i++) {
                if (dataBlocks.get(i).category.equals("source")) {
                    front.add(dataBlocks.get(i));
                    //System.out.println("zaradil som block:" + i + "na koniec frontu");
                }
            }
            while (!front.isEmpty()) {
                DataBlock temp = front.poll();
                if (temp.execute()) {
                    temp.sendOutputs();
                }
                for (int i = 0; i < datovods.size(); i++) {
                    //System.out.println("datovods : " + datovods.elementAt(i));
                    Datovod d = datovods.elementAt(i);
                    int startIndex = d.getStartPoint().getBlockId();
                    int startInputId = d.getStartPoint().getOutputId();
                    int endIndex = d.getEndPoint().getBlockId();
                    int endOutputId = d.getEndPoint().getInputId();
                    if (dataBlocks.get(startIndex).equals(temp)) {
                        if (!front.contains(dataBlocks.get(endIndex))) {
                            front.add(dataBlocks.get(endIndex));
                            //System.out.println("zaradil som block:" + endIndex + "na koniec frontu");
                        }
                    }
                }
                if (temp.hasInputs()) {
                    front.add(temp);
                    //System.out.println("zaradil som spracovavany block na koniec frontu");
                }
            }
            for (int i = 0; i < dataBlocks.size(); i++) {
                dataBlocks.elementAt(i).finalizeBlockWork();
            }

        }
    }

    public void SaveConversion(String path) {
        try {

            DocumentBuilderFactory docFactory = DocumentBuilderFactory.newInstance();
            DocumentBuilder docBuilder = docFactory.newDocumentBuilder();

            //root elements
            Document doc = docBuilder.newDocument();
            Element rootElement = doc.createElement("Conversion");
            doc.appendChild(rootElement);

            //staff elements
            Element XMLblocks = doc.createElement("blocks");
            rootElement.appendChild(XMLblocks);
            Element XMLDatovods = doc.createElement("connectors");
            rootElement.appendChild(XMLDatovods);

            for (int i = 0; i < gbloks.size(); i++) {
                GraphicBlock temp = gbloks.elementAt(i);

                Element b = doc.createElement("block");
                Attr id = doc.createAttribute("id");
                id.setValue(String.valueOf(i));
                b.setAttributeNode(id);

                Attr X = doc.createAttribute("X");
                X.setValue(String.valueOf(temp.getX()));
                b.setAttributeNode(X);

                Attr Y = doc.createAttribute("Y");
                Y.setValue(String.valueOf(temp.getY()));
                b.setAttributeNode(Y);
                XMLblocks.appendChild(gbloks.elementAt(i).getDataBlock().getXML(b, doc));
            }
            for (int i = 0; i < datovods.size(); i++) {
                //System.out.println("datovods : " + datovods.elementAt(i));
                Datovod temp = datovods.elementAt(i);
                int startIndex = temp.getStartPoint().getBlockId();
                int startOuputId = temp.getStartPoint().getOutputId();
                int endIndex = temp.getEndPoint().getBlockId();
                int endInputId = temp.getEndPoint().getInputId();
                Element connector = doc.createElement("connector");
                Attr from = doc.createAttribute("from");
                from.setValue(String.valueOf(startIndex));
                connector.setAttributeNode(from);
                Attr outputId = doc.createAttribute("outputId");
                outputId.setValue(String.valueOf(startOuputId));
                connector.setAttributeNode(outputId);
                Attr to = doc.createAttribute("to");
                to.setValue(String.valueOf(endIndex));
                connector.setAttributeNode(to);
                Attr inputId = doc.createAttribute("inputId");
                inputId.setValue(String.valueOf(endInputId));
                connector.setAttributeNode(inputId);
                Attr type = doc.createAttribute("type");
                String typeString = "";
                if (temp.getType() == 0) {
                    typeString = "Unlimited";
                } else {
                    typeString = "Limited";
                }
                type.setValue(typeString);
                connector.setAttributeNode(type);
                XMLDatovods.appendChild(connector);
            }


            doc.getDocumentElement().normalize();
            doc.setXmlStandalone(true);


            //write the content into xml file
            TransformerFactory transformerFactory = TransformerFactory.newInstance();
            Transformer transformer = transformerFactory.newTransformer();
            //transformer.setOutputProperty(OutputKeys.INDENT, "yes");
            DOMSource source = new DOMSource(doc.getDocumentElement());
            StreamResult result = new StreamResult(new File(path));
            //StreamResult result =  new StreamResult(System.out);
            transformer.transform(source, result);

            //System.out.println("Done");

        } catch (ParserConfigurationException pce) {
            pce.printStackTrace();
        } catch (TransformerException tfe) {
            tfe.printStackTrace();
        }
    }

    private void loadConversion(File file, GraphicCanvas canvas) {
        try {
            DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
            DocumentBuilder db = dbf.newDocumentBuilder();
            Document doc = db.parse(file);
            doc.getDocumentElement().normalize();
            //System.out.println("Root element " + doc.getDocumentElement().getNodeName());
            NodeList nodeLst = doc.getElementsByTagName("block");

            for (int s = 0; s < nodeLst.getLength(); s++) {

                Node fstNode = nodeLst.item(s);

                if (fstNode.getNodeType() == Node.ELEMENT_NODE) {
                    Element fstElmnt = (Element) fstNode;
                    String type = fstElmnt.getAttribute("type");
                    String[] types = Config.blockTypes;
                    String[][] blocks = Config.blocks;
                    String className = "";
                    for (int i = 0; i < types.length; i++) {
                        // blocky
                        String[] content = blocks[i];
                        for (int j = 0; j < content.length; j++) {
                            if (type.equals(content[j])) {
                                className = types[i].toLowerCase() + "." + content[j];
                            }
                        }
                    }
                    DataBlock dataBlock = null;
                    Class[] classParm = null;
                    Object[] objectParm = null;
                    try {
                        String name = "baka.dataflow.blocks." + className;
                        Class cl = Class.forName(name);
                        java.lang.reflect.Constructor co = cl.getConstructor(classParm);
                        dataBlock = (DataBlock) co.newInstance(objectParm);
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                    dataBlock.loadXML(fstElmnt);
                    this.dataBlocks.add(dataBlock);
                    GraphicBlock temp = new GraphicBlock(Integer.parseInt(fstElmnt.getAttribute("id")), Config.BlockWidth, Config.BlockHeight, dataBlock, canvas);
                    temp.setBounds(Integer.parseInt(fstElmnt.getAttribute("X")), Integer.parseInt(fstElmnt.getAttribute("Y")), Config.BlockWidth, Config.BlockHeight);
                    dataBlock.setGraphicBlok(temp);
                    dataBlock.setLoadedSettings();
                    gbloks.add(temp);
                    //System.out.println("className" + className);
                    //System.out.println("Element id:" + fstElmnt.getAttribute("id") + " type:" + fstElmnt.getAttribute("type"));
                }

            }
            NodeList connectorLst = doc.getElementsByTagName("connector");
            for (int s = 0; s < connectorLst.getLength(); s++) {

                Node fstNode = connectorLst.item(s);

                if (fstNode.getNodeType() == Node.ELEMENT_NODE) {

                    Element fstElmnt = (Element) fstNode;
                    int outputBlokId = Integer.parseInt(fstElmnt.getAttribute("from"));
                    int inputBlokId = Integer.parseInt(fstElmnt.getAttribute("to"));
                    int outputId = Integer.parseInt(fstElmnt.getAttribute("outputId"));
                    int inputId = Integer.parseInt(fstElmnt.getAttribute("inputId"));
                    String type = fstElmnt.getAttribute("type");
                    DatovodPoint start = new DatovodPoint(outputBlokId, -1, outputId);
                    DatovodPoint end = new DatovodPoint(inputBlokId, inputId, -1);
                    int typeId = (type.equals("Unlimited")) ? 0 : 1;
                    Datovod d = new Datovod(start, end, typeId);
                    this.datovods.add(d);
                    //System.out.println("Connector from:" + fstElmnt.getAttribute("from") + " to:" + fstElmnt.getAttribute("to"));
                }

            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    public Vector<GraphicBlock> getGbloks() {
        return gbloks;
    }

    public Vector<Datovod> getDatovods() {
        return datovods;
    }

    public String getInterpreterInformations() {
        int source = 0;
        int target = 0;
        String sourceT = "";
        String targerT = "";
        for (int i = 0; i < this.dataBlocks.size(); i++) {
            DataBlock temp = this.dataBlocks.elementAt(i);
            if (temp.category.equals("source")) {
                source++;
                if (!sourceT.equals("")) {
                    sourceT += ",";
                }
                sourceT += "id:" + i;
            }
            if (temp.category.equals("target")) {
                target++;
                if (!targerT.equals("")) {
                    targerT += ",";
                }
                targerT += "id:" + i;
            }
        }
        return source+" source("+sourceT+") and "+target+" target("+targerT+") blocks.";
    }

    public void setFileLocation(int id, String location){
        DataBlock temp = this.dataBlocks.elementAt(id);
        temp.setBlockFile(location);
    }
}
