package baka.dataflow.blocks.manipulation;

import baka.dataflow.DataBlock;
import java.awt.event.ActionEvent;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 *
 * @author Adam Poldauf
 */
public class Cutter extends DataBlock {

    String startSequence;
    String endSequence;
    JTextField startField;
    JTextField endField;
    boolean keepSequences = false;
    JCheckBox keepSequencesCheckbox;

    public Cutter() {
        this.type = "Cutter";
        this.category = "manipulation";
        this.description = "Cuts substring from input defined by start and end sequence.";
        constructProperties();
        this.propertiesPanel.setName(this.type);


        startField = new JTextField(startSequence);
        //startField.addActionListener(this);
        startField.getDocument().addDocumentListener(new DocumentListener() {

            public void insertUpdate(DocumentEvent e) {
                change();
            }

            public void removeUpdate(DocumentEvent e) {
                change();
            }

            public void changedUpdate(DocumentEvent e) {
                change();
            }

            public void change() {
                startSequence = startField.getText();
            }
        });
        this.propertiesPanel.add(this.createPropertiesPanelRow(new JLabel("Start Sequence"), startField));

        endField = new JTextField(endSequence);
        //endField.addActionListener(this);
        endField.getDocument().addDocumentListener(new DocumentListener() {

            public void insertUpdate(DocumentEvent e) {
                change();
            }

            public void removeUpdate(DocumentEvent e) {
                change();
            }

            public void changedUpdate(DocumentEvent e) {
                change();
            }

            public void change() {
                endSequence = endField.getText();
            }
        });
        this.propertiesPanel.add(this.createPropertiesPanelRow(new JLabel("End Sequence"), endField));

        this.keepSequencesCheckbox = new JCheckBox();
        this.keepSequencesCheckbox.setActionCommand("keepSequences");
        this.keepSequencesCheckbox.addActionListener(this);
        this.propertiesPanel.add(this.createPropertiesPanelRow(new JLabel("keep Sequences"), keepSequencesCheckbox));
    }

    @Override
    public void processInput() {
        String line = "";
        if ((line = inputBus.readSpecific(0)) != null) {
            if (line.contains(startSequence) && line.contains(endSequence)) {
               // System.out.println("line : " + line + " obsahuje hladany podretazec");
                while (line.contains(startSequence) && line.contains(endSequence)) {
                   // System.out.println("line : " + line + " STALE obsahuje hladany podretazec");
                    int subStart = line.indexOf(startSequence);
                    int subEnd = line.indexOf(endSequence);
                    //System.out.println("start index :" + subStart);
                    //System.out.println("end index :" + subEnd);
                    while (subEnd <= (subStart + startSequence.length())) {
                        //System.out.println("hladam iny ENDINDEX");
                        subEnd = line.indexOf(endSequence, subEnd + 1);
                        //System.out.println("end v hldani : " + subEnd);
                    }
                    //System.out.println("NAISEL SOM INY :" + subEnd);
                    subEnd += endSequence.length();
                    String output = line.substring(subStart, subEnd);
//                    System.out.println("CUTTER POVODNY OUTPUT:"+output);
                    String result="";
                    if(!keepSequences){
                        result = output.substring(startSequence.length(),output.length()-endSequence.length());
                    }else{
                        result = output;
                    }
//                    System.out.println("CUTTER OUTPUT:"+output);
//                    System.out.println("StartSequence:"+startSequence);
//                    System.out.println("EndSequence:"+endSequence);
                    outputBus.writeToAll(result);
                    line = line.substring(subEnd);
                }
            }
        }
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        super.actionPerformed(e);
        if (e.getActionCommand().equals("keepSequences")) {
            this.keepSequences = this.keepSequencesCheckbox.isSelected();
        }
    }


    @Override
    public Element getXML(Element e, Document doc) {
        super.getXML(e, doc);
        Attr startSq = doc.createAttribute("startSq");
        startSq.setValue(this.startSequence);
        e.setAttributeNode(startSq);

        Attr endSq = doc.createAttribute("endSq");
        endSq.setValue(this.endSequence);
        e.setAttributeNode(endSq);

        Attr keepSequencesA = doc.createAttribute("keepSequences");
        keepSequencesA.setValue(String.valueOf(this.keepSequences));
        e.setAttributeNode(keepSequencesA);
        return e;
    }

    @Override
    public void loadXML(Element e) {
        super.loadXML(e);
        this.startSequence = e.getAttribute("startSq");
        this.endSequence = e.getAttribute("endSq");
        String keepSequencesA = e.getAttribute("keepSequences");
        if (keepSequencesA.equals("true")) {
            this.keepSequences = true;
        } else {
            this.keepSequences = false;
        }
    }

    @Override
    public void setLoadedSettings() {
        super.setLoadedSettings();
        this.startField.setText(this.startSequence);
        this.endField.setText(this.endSequence);
        this.keepSequencesCheckbox.setSelected(this.keepSequences);
    }

    @Override
    public void resetBlockState() {
    }

    @Override
    public void finalizeBlockWork() {
       
    }
}
