/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package baka.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Vector;

/**
 *
 * @author madda
 */
class GraphicCanvasMouseAdapter extends MouseAdapter {

    GraphicCanvas owner;
    GraphicBlock dragedBlok = null;
    Rectangle dragedArea = null;
    Point dragStart, dragEnd, lastPosition = null;
    Vector<GraphicBlock> selectedBlocks;

    public GraphicCanvasMouseAdapter(GraphicCanvas parent) {
        this.owner = parent;
        selectedBlocks = new Vector<GraphicBlock>();
    }

    @Override
    public void mousePressed(MouseEvent e) {
        Point clickedPoint = e.getPoint();
        int id = owner.getClickedBlock(clickedPoint);
        if (id != -1) {
            this.dragedBlok = owner.blocks.get(id);
            lastPosition = e.getPoint();
        } else if (owner.selectedArea != null) {
            if (owner.selectedArea.contains(clickedPoint)) {
                System.out.println("klik do plochy" + e.getPoint());
                lastPosition = e.getPoint();
                dragedArea = owner.selectedArea;
                selectedBlocks.clear();
                for (int i = 0; i < owner.blocks.size(); i++) {
                    GraphicBlock gb = owner.blocks.get(i);
                    Rectangle rect = gb.getBounds();
                    if (dragedArea.contains(rect)) {
                        selectedBlocks.add(gb);
                    }
                }
            } else {
                System.out.println("klik mimo plochy");
                owner.selectedArea = null;
                dragedArea = null;
                selectedBlocks.clear();
            }
        }
    }

    @Override
    public void mouseReleased(MouseEvent e) {
        if (dragedBlok == null) {
            if ((dragStart != null) && (dragEnd != null)) {
                Rectangle place = new Rectangle(dragStart);
                place.add(dragEnd);
                System.out.println("Place:" + place);
                owner.selectedArea = place;
            }
            this.dragStart = null;
            this.dragEnd = null;
            dragedArea = null;
        } else {
            this.dragedBlok = null;
        }
        owner.repaint();
    }

    @Override
    public void mouseDragged(MouseEvent e) {
        if (this.dragedBlok != null) {
            Rectangle rect = this.dragedBlok.getBounds();
            int deltaX = e.getPoint().x - lastPosition.x;
            int deltaY = e.getPoint().y - lastPosition.y;
            int x = rect.x + deltaX;
            int y = rect.y + deltaY;
            if (x < 0) {
                x = 0;
            }
            if (y < 0) {
                y = 0;
            }
            boolean changed = false;
            Dimension area = owner.getSize();

            int this_width = x + 100;
            if (this_width > area.width) {
                area.width = this_width;
                changed = true;
            }
            int width1 = x - 5;
            if (width1 < 0) {
                area.width += 50;
                changed = true;
            }

            int height1 = y - 5;
            if (height1 < 0) {
                area.height += 50;
                changed = true;
            }

            int this_height = y + 100;
            if (this_height > area.height) {
                area.height = this_height;
                changed = true;
            }
            if (changed) {
                owner.setPreferredSize(area);
                owner.revalidate();
            }
            Rectangle rectangle = new Rectangle(x, y, 50, 50);
            owner.scrollRectToVisible(rectangle);
            lastPosition = e.getPoint();
            this.dragedBlok.setLocation(x, y);
            owner.repaint();
        } else {
            if (dragedArea != null) {
                // posuvam selected areu
                dragedArea = owner.selectedArea;
                //System.out.println("draged area" + e.getPoint());
                int deltaX = e.getPoint().x - lastPosition.x;
                int deltaY = e.getPoint().y - lastPosition.y;
                int x = dragedArea.x + deltaX;
                int y = dragedArea.y + deltaY;
                boolean changed = false;
                Dimension area = owner.getSize();
                if (x < 0) {
                    x = 0;
                }
                if (y < 0) {
                    y = 0;
                }
                int this_width = x + dragedArea.width;
                if (this_width > area.width) {
                    area.width = this_width;
                    changed = true;
                }
                int width1 = x - 5;
                if (width1 < 0) {
                    area.width += 100;
                    changed = true;
                }

                int height1 = y - 5;
                if (height1 < 0) {
                    area.height += 100;
                    changed = true;
                }

                int this_height = y + dragedArea.height + 50;
                if (this_height > area.height) {
                    area.height = this_height;
                    changed = true;
                }
                if (changed) {
                    owner.setPreferredSize(area);
                    owner.revalidate();
                }
                Rectangle oldAreaPosition = dragedArea.getBounds();
                owner.selectedArea.setLocation(x, y);
                owner.scrollRectToVisible(owner.selectedArea);
                for (int i = 0; i < selectedBlocks.size(); i++) {
                    GraphicBlock gb = selectedBlocks.get(i);
                    Rectangle rect = gb.getBounds();
                    int XinArea = rect.x - oldAreaPosition.x;
                    int YinArea = rect.y - oldAreaPosition.y;
                    gb.setLocation(x + XinArea, y + YinArea);
                }
                lastPosition = e.getPoint();
                owner.repaint();
            } else {
                // vytvaram novu selected areau
                if (dragStart == null) {
                    dragStart = e.getPoint();
                } else {
                    dragEnd = e.getPoint();
                }
            }
        }

    }

    @Override
    public void mouseClicked(MouseEvent e) {
        owner.requestFocusInWindow();
        Point clickedPoint = e.getPoint();
        int id = owner.getClickedBlock(clickedPoint);
        if (id != -1) {
            GraphicBlock temp = owner.blocks.get(id);
            owner.selectedBlock = temp;
            Point location = temp.getLocation();
            int blockX = clickedPoint.x - location.x;
            int blockY = clickedPoint.y - location.y;
            Point blockPoint = new Point(blockX, blockY);
            int inID = temp.isInputPoint(blockPoint);


            int outID = temp.isOutputPoint(blockPoint);

            if (inID > -1) {
                // prvy klinuty input
                owner.lastClickedInput = new DatovodPoint(id, inID, -1);
            } else if (outID > -1) {
                owner.lastClickedOutput = new DatovodPoint(id, -1, outID);
            }

            if ((owner.lastClickedInput.isEmpty() == false) && (owner.lastClickedOutput.isEmpty() == false)) {
                if (owner.lastClickedInput.blockId != owner.lastClickedOutput.blockId) {
                    Datovod d = new Datovod(owner.lastClickedOutput, owner.lastClickedInput, owner.parent.DatovodType);
                    if (owner.hasConnector(d)) {
                        System.out.println("Remove datovod.");
                        owner.connectors.remove(d);
                    } else {
                        // kontrola je input aj output volny
                        if (owner.availableConnector(d)) {
                            owner.connectors.add(d);
                        }
                    }
                }
                owner.lastClickedInput = new DatovodPoint();
                owner.lastClickedOutput = new DatovodPoint();
            }

            owner.parent.settings.removeAll();
            Component properties = temp.getDataBlock().getPropertiesPanel();
            owner.parent.settings.add(properties, BorderLayout.CENTER);
            owner.parent.settings.validate();
            owner.parent.settings.repaint();
        } else {
            owner.selectedBlock = null;
            owner.lastClickedInput = new DatovodPoint();
            owner.lastClickedOutput = new DatovodPoint();
        }
        owner.repaint();


    }
}
