/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package baka.gui;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.util.TooManyListenersException;
import javax.swing.*;
import baka.dataflow.*;
import baka.main.Config;
import java.awt.event.ActionListener;
import java.io.File;
import javax.swing.filechooser.FileFilter;

/**
 * Hlavna trieda bakalarskej prace.
 * @author Adam Poldauf
 */
public class VisualFrame extends JFrame implements ActionListener {

    JTabbedPane left;
    JTabbedPane tabs;
    GraphicCanvas main;
    JPanel bottom;
    JPanel settings;
    JTextArea texts;
    AbstractAction saveAction, loadAction, runAction;
    int DatovodType = 0;
    int ConversionType = 0;
    XMLFileFiler fileFilter = new XMLFileFiler();

    public VisualFrame() throws TooManyListenersException {
        // title
        super("Visual file processing");
        // set system look and feel
        try {
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
        } catch (Exception e) {
            e.printStackTrace();
        }

        // default close operation
        this.setDefaultCloseOperation(EXIT_ON_CLOSE);

        // setting layout manager 
        Container contentPane = this.getContentPane();
        contentPane.setLayout(new BorderLayout());


        //Actions
        initializeActions();

        // Menu
        this.setJMenuBar(createMenuBar());

        // Toolbar
        this.add(createToolBar(), BorderLayout.PAGE_START);

        // TransferHandler
        TransferHandler t = new BlockTransferHandler();

        // Blok Pallete
        left = createBlockPallete(t);
        // create main area
        main = new GraphicCanvas(this);
        main.setTransferHandler(t);
        main.getDropTarget().addDropTargetListener(new GraphicCanvasDropTargetListener(main));
        JScrollPane scroller = new JScrollPane(main);
        scroller.setPreferredSize(new Dimension(800, 600));
        tabs = new JTabbedPane();
        tabs.addTab("Conversion 1", scroller);
        //this.add(createPanelForComponent(tabs, "Conversion"), BorderLayout.CENTER);

        JPanel center = new JPanel(new BorderLayout(5, 5));
        center.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
        center.add(left, BorderLayout.WEST);
        center.add(tabs, BorderLayout.CENTER);
        this.add(center, BorderLayout.CENTER);


        // Bottom panel for sett & text messages
        bottom = new JPanel(new BorderLayout());
        bottom.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));

        texts = new JTextArea(8, 50);
        //texts.setText("Welcome to Visual File Language.");
        texts.setEditable(false);
        JScrollPane textScroller = new JScrollPane(texts);
        bottom.add(createPanelForComponent(textScroller, "Texts"), BorderLayout.CENTER);

        settings = new JPanel(new BorderLayout());
        settings.setPreferredSize(new Dimension(450, 0));
        JLabel name = new JLabel("No Block selected.");
        settings.add(name);
        bottom.add(createPanelForComponent(settings, "Blok settings"), BorderLayout.WEST);

        this.add(bottom, BorderLayout.SOUTH);


        // sirka okna vlastna alebo prisposobena komponentom
        //this.setSize(180, 80);
        this.pack();
        this.setVisible(true);
    }

    private void initializeActions() {
        ImageIcon saveIcon = createImageIcon("icons/save-icon.gif", null);
        saveAction = new SaveAction(null, saveIcon, "Save Conversion", KeyEvent.VK_S);
        ImageIcon openIcon = createImageIcon("icons/icon-open.gif", null);
        loadAction = new LoadAction(null, openIcon, "Open Conversion", KeyEvent.VK_O);
        ImageIcon runIcon = createImageIcon("icons/run-icon.gif", null);
        runAction = new RunAction(null, runIcon, "Execute Conversion", null);
    }

    private JTabbedPane createBlockPallete(TransferHandler t) {
        JTabbedPane ret = new JTabbedPane();
        String[] types = Config.blockTypes;
        String[][] blocks = Config.blocks;
        for (int i = 0; i < types.length; i++) {
            JPanel temp = new JPanel();
            temp.setPreferredSize(new Dimension(150, 0));
            //temp.setBackground(Color.white);
            //temp.setBorder(BorderFactory.createLineBorder(Color.BLACK));
            ret.addTab(types[i], null, temp, types[i]);
            // blocky
            String[] content = blocks[i];
            for (int j = 0; j < content.length; j++) {
                String name = types[i].toLowerCase() + "." + content[j];
                PalleteBlock b = new PalleteBlock(name);
                b.setTransferHandler(t);
                temp.add(b);
                Class[] classParm = null;
                Object[] objectParm = null;

                DataBlock dataBlock = null;
                try {
                    String clasName = "baka.dataflow.blocks." + name;
                    Class cl = Class.forName(clasName);
                    java.lang.reflect.Constructor co = cl.getConstructor(classParm);
                    dataBlock = (DataBlock) co.newInstance(objectParm);
                } catch (Exception e) {
                    e.printStackTrace();
                }
                b.setToolTipText(dataBlock.description);
            }
        }
        return ret;
    }

    private JToolBar createToolBar() {
        JToolBar tb = new JToolBar();

        JPanel toolbarPanel = new JPanel(new BorderLayout());

        JPanel cardPanel = new JPanel(new GridLayout(0, 2, 15, 0));

        toolbarPanel.add(cardPanel, BorderLayout.NORTH);

        JButton l = new JButton(loadAction);
        l.setRequestFocusEnabled(false);
        l.setFocusPainted(false);
        tb.add(l);

        JButton c = new JButton(saveAction);
        c.setRequestFocusEnabled(false);
        tb.add(c);



        JButton b;

        b = new JButton(runAction);
        b.setRequestFocusEnabled(false);
        tb.add(b);


        AbstractAction limitedK = new changeKabel("Limited", null, "Limited");
        AbstractAction unlimitedK = new changeKabel("Unlimited", null, "Unlimited");
        AbstractAction memoryK = new changeKabel("Memory", null, "Memory");
        JRadioButton limitedKabel = new JRadioButton(limitedK);
        JRadioButton unlimitedKabel = new JRadioButton(unlimitedK);
        JRadioButton memoryKabel = new JRadioButton(memoryK);
        unlimitedKabel.setSelected(true);


        JPanel kabelSelection = new JPanel();
        kabelSelection.setBorder(BorderFactory.createTitledBorder("Datovod Type"));
        ButtonGroup kabelGroup = new ButtonGroup();
        kabelGroup.add(limitedKabel);
        kabelGroup.add(unlimitedKabel);
        kabelGroup.add(memoryKabel);
        kabelSelection.add(unlimitedKabel);
        kabelSelection.add(limitedKabel);
        kabelSelection.add(memoryKabel);
        //tb.add(createPanelForComponent(kabelSelection, "Datavod"));
        JPanel widthFix = new JPanel(new BorderLayout());
        widthFix.add(kabelSelection, BorderLayout.EAST);
        cardPanel.add(widthFix);

        AbstractAction synchronousC = new changeConversion("Synchronous", null, "Synchronous");
        AbstractAction asynchronousC = new changeConversion("Asynchronous", null, "Asynchronous");

        JRadioButton synchronConversion = new JRadioButton(synchronousC);
        JRadioButton asynchronConversion = new JRadioButton(asynchronousC);
        synchronConversion.setSelected(true);
        JPanel conversionSelection = new JPanel();
        conversionSelection.setBorder(BorderFactory.createTitledBorder("Conversion type"));
        ButtonGroup conversionGroup = new ButtonGroup();
        conversionGroup.add(synchronConversion);
        conversionGroup.add(asynchronConversion);
        conversionSelection.add(synchronConversion);
        conversionSelection.add(asynchronConversion);

        //tb.add(createPanelForComponent(conversionSelection, "Conversion type"));
        JPanel widthFix1 = new JPanel(new BorderLayout());
        widthFix1.add(conversionSelection, BorderLayout.EAST);
        cardPanel.add(widthFix1);

        JPanel widthFix2 = new JPanel(new BorderLayout());
        widthFix2.add(toolbarPanel, BorderLayout.WEST);
        tb.add(widthFix2);
        tb.setFloatable(false);
        return tb;
    }

    private JMenuBar createMenuBar() {
        JMenuBar mb = new JMenuBar();

        JMenu file = new JMenu("File");
        file.setMnemonic(KeyEvent.VK_F);
        mb.add(file);

        JMenuItem newMenuItem = new JMenuItem("New");
        newMenuItem.setAccelerator(KeyStroke.getKeyStroke(
                KeyEvent.VK_N, ActionEvent.CTRL_MASK));

        newMenuItem.setActionCommand("New");
        newMenuItem.setMnemonic(KeyEvent.VK_N);
        newMenuItem.addActionListener(this);
        file.add(newMenuItem);

        file.addSeparator();



        JMenuItem savetMenuItem = new JMenuItem(new SaveAction("Save", null, "Save Conversion", KeyEvent.VK_S));
        savetMenuItem.setAccelerator(KeyStroke.getKeyStroke(
                KeyEvent.VK_S, ActionEvent.CTRL_MASK));
//        savetMenuItem.setActionCommand("Save");
//        savetMenuItem.setMnemonic(KeyEvent.VK_S);
//        savetMenuItem.addActionListener(this);
        file.add(savetMenuItem);


        JMenuItem loadMenuItem = new JMenuItem(new LoadAction("Open", null, "Open Conversion", KeyEvent.VK_O));
        loadMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, ActionEvent.CTRL_MASK));
//        loadMenuItem.setActionCommand("Load");
//        loadMenuItem.setMnemonic(KeyEvent.VK_O);
//        loadMenuItem.addActionListener(this);
        file.add(loadMenuItem);

        file.addSeparator();

        //ImageIcon icon = createImageIcon("icons/exit.gif","Icon");

        JMenuItem exitMenuItem = new JMenuItem("Exit");
        exitMenuItem.setActionCommand("Exit");
        exitMenuItem.setMnemonic(KeyEvent.VK_E);
        exitMenuItem.addActionListener(this);
        file.add(exitMenuItem);

        return mb;
    }

    private JPanel createPanelForComponent(JComponent comp,
            String title) {
        JPanel panel = new JPanel(new BorderLayout());
        panel.add(comp, BorderLayout.CENTER);
        if (title != null) {
            panel.setBorder(BorderFactory.createTitledBorder(title));
        } else {
            panel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
        }
        return panel;
    }

    public void actionPerformed(ActionEvent e) {
        if (e.getSource() instanceof JMenuItem) {
            JMenuItem item = (JMenuItem) e.getSource();
            String cmd = item.getActionCommand();
            if (cmd.equals("Exit")) {
                System.exit(0);
            } else if (cmd.equals("New")) {
                main.clearCanvas();
                JLabel name = new JLabel("No Block selected.");
                settings.removeAll();
                settings.add(name);
                settings.validate();
                settings.repaint();
            }
        }
    }

    protected ImageIcon createImageIcon(String path, String description) {
        java.net.URL imgURL = getClass().getResource(path);
        if (imgURL != null) {
            return new ImageIcon(imgURL, description);
        } else {
            System.err.println("Couldn't find file: " + path);
            return null;
        }
    }



    class SaveAction extends AbstractAction {

        public SaveAction(String text,
                ImageIcon icon,
                String desc,
                Integer mnemonic) {

            super(text, icon);

            putValue(SHORT_DESCRIPTION, desc);

            putValue(MNEMONIC_KEY, mnemonic);

        }

        public void actionPerformed(ActionEvent e) {
            //throw new UnsupportedOperationException("Not supported yet.");
            System.out.println("Save Action");
            JFileChooser fc = new JFileChooser();
            fc.setFileFilter(fileFilter);

            int returnVal = fc.showSaveDialog(null);

            if (returnVal == JFileChooser.APPROVE_OPTION) {
                File file = fc.getSelectedFile();
                String filePath = file.getPath();
                String fileName = file.getName();
                if (!fileName.endsWith(".xml")) {
                    filePath += ".xml";
                }
                Conversion c = new Conversion(main.blocks, main.connectors,ConversionType);
                c.SaveConversion(filePath);
                texts.append("Conversion successfully saved.\n");
            }
        }
    }

    class RunAction extends AbstractAction {

        public RunAction(String text,
                ImageIcon icon,
                String desc,
                Integer mnemonic) {

            super(text, icon);

            putValue(SHORT_DESCRIPTION, desc);

            putValue(MNEMONIC_KEY, mnemonic);

        }

        public void actionPerformed(ActionEvent e) {
            texts.append("Runinng conversion. Please wait.\n");
            Conversion c = new Conversion(main.blocks, main.connectors, ConversionType);
            c.executeConversion();
            texts.append("Conversion successfully executed.\n");
        }
    }

    class LoadAction extends AbstractAction {

        public LoadAction(String text,
                ImageIcon icon,
                String desc,
                Integer mnemonic) {

            super(text, icon);

            putValue(SHORT_DESCRIPTION, desc);

            putValue(MNEMONIC_KEY, mnemonic);

        }

        public void actionPerformed(ActionEvent e) {
            System.out.println("Load Action");
            JFileChooser fc = new JFileChooser();
            fc.setFileFilter(fileFilter);

            int returnVal = fc.showOpenDialog(null);

            if (returnVal == JFileChooser.APPROVE_OPTION) {
                File file = fc.getSelectedFile();
                String filePath = file.getPath();
                String fileName = file.getName();
                Conversion c = new Conversion(file,main);
                main.clearCanvas();
                JLabel name = new JLabel("No Block selected.");
                settings.removeAll();
                settings.add(name);
                settings.validate();
                settings.repaint();
                main.load(c.getGbloks(), c.getDatovods());
                texts.append("Conversion successfully loaded " + fileName + ".\n");
            }

        }
    }

    class changeKabel extends AbstractAction {

        public changeKabel(String text,
                ImageIcon icon,
                String desc) {

            super(text, icon);

            putValue(SHORT_DESCRIPTION, desc);

        }

        public void actionPerformed(ActionEvent e) {
            String type = e.getActionCommand();
            if (type.equals("Limited")) {
                texts.append("Datovod changed to Limited.\n");
                DatovodType = 1;
            } else if(type.equals("Unlimited")) {
                texts.append("Datovod changed to Unlimited.\n");
                DatovodType = 0;
            }else if(type.equals("Memory")) {
                texts.append("Datovod changed to Memory.\n");
                DatovodType = 2;
            }
        }
    }

    class changeConversion extends AbstractAction {

        public changeConversion(String text,
                ImageIcon icon,
                String desc) {

            super(text, icon);

            putValue(SHORT_DESCRIPTION, desc);

        }

        public void actionPerformed(ActionEvent e) {
            String type = e.getActionCommand();
            if (type.equals("Synchronous")) {
                texts.append("Conversion changed to Synchronous.\n");
                ConversionType = 0;
            } else {
                texts.append("Conversion changed to Asynchronous.\n");
                ConversionType = 1;
            }
        }
    }

    class XMLFileFiler extends FileFilter {

        @Override
        public boolean accept(File f) {
            if (f.isDirectory()) {
                return true;
            }

            String s = f.getName();

            return s.endsWith(".xml") || s.endsWith(".XML");

        }

        @Override
        public String getDescription() {
            return ".xml";
        }
    }
}
